import turtle, math


def  main():
    # put label on top of page
    turtle.title ('Graphs of Functions')

    # setup screen size
    turtle.setup(1000, 1000, 0, 0)

    # create a turtle object
    t = turtle.Turtle()
    t.speed(0)
    t.ht()

    # draw the grid
    drawGridScaled(t)

    # draw sine finction
    t.pencolor('red')
    drawFnScaled(t, math.sin, -math.pi, math.pi, 0.01)
    t.write('y = sin(x)  [in radians]')

    # draw cosine function
    t.pencolor('blue')
    drawFnScaled(t, math.cos, -math.pi, math.pi, 0.01)
    t.write('y = cos(x)  [in radians]')

    # draw my function 1
    t.pencolor('purple')
    drawFnScaled(t, myFunc1, -2.8, 2.8, 0.01)
    t.write('y = x * x - 4')

    # draw my function 2
    t.pencolor('green')
    drawFnScaled(t, myFunc2, -2, 2, 0.01)
    t.write('y = x ^ 3 / 2')

    turtle.exitonclick()


def drawLine(t, x1, y1, x2, y2):
# draw a line from (x1, y1) to (x2, y2)
    t.penup()
    t.goto(x1, y1)
    t.pendown()
    t.goto(x2, y2)
    t.penup()


def labelPoint(t, x, y, label):
# write a label at location x, y
    t.penup()
    t.goto(x, y)
    t.pendown()
    t.write(label)
    t.penup()


def drawGridMark(t, x, y, isVertical):
    if isVertical:
        drawLine(t, x, y + 5, x, y - 5)
    else:
        drawLine(t, x - 5, y, x + 5, y)


def labelGridPoint(t, x, y, isVertical, text):
    if isVertical:
        labelPoint(t, x - 20, y - 20, text)
    else:
        labelPoint(t, x + 20, y, text)


def drawGridScaled(t):
# draw the axes
    drawLine(t, -450, 0, 450, 0)
    drawLine(t, 0, 450, 0, -450)

# label x-axis
    for x in [-400,-350,-300,-250,-200,-150,-100,-50,0,50,100,150,200,250,300,350,400]:
        drawGridMark(t, x, 0, True)
        labelGridPoint(t, x, 0, True, (x/100, 0))

# label y-axis
    for y in [-400,-350,-300,-250,-200,-150,-100,-50,0,50,100,150,200,250,300,350,400]:
        drawGridMark(t, 0, y, False)
        labelGridPoint(t, 0, y, False, (0, y/100))


def drawFnScaled(t, fn, lower, upper, step):
    t.penup()
    x = lower
    y = fn(x)
    scaledX = x * 100
    scaledY = y * 100
    t.goto(scaledX, scaledY)
    t.pendown()
    while x < upper:
        x = x + step
        y = fn(x)
        scaledX, scaledY = x * 100, y * 100
        t.goto(scaledX, scaledY)
    t.penup()


def myFunc1(x):
    return (x ** 2 - 4)


def myFunc2(x):
    return (x ** 3 / 2)


if __name__ == '__main__':
    main()
