import turtle
import TurtleUtilsPMP as tp


def main():
    scr = tp.initializescreen()
    t1 = tp.initializeturtle()

    t1.pensize(4)
    t1.pu()
    t1.setpos(200, 200)
    taijitu(t1, 200, 'blue', 'red')
    t1.pu()
    t1.setpos(-150, -150)
    taijitu(t1, 100, 'green', 'purple')
    t1.pu()
    t1.setpos(350, -200)
    taijitu(t1, 150, 'black', 'white')
    t1.ht()

    scr.exitonclick()


def taijitu(t, r, col1, col2): 
    '''
    Draw a classic Taoist taijitu of the given radius centered on the current
    turtle position. The 'eyes' are placed along the turtle's heading, the
    filled one in front, the open one behind.
    '''
 
    # useful derivative values
    r2, r4, r8 = (r >> s for s in (1, 2, 3))
 
    # remember where we started
    x0, y0 = start = t.pos()
    startcolour = t.color()
    startheading = t.heading()

    # draw outer circle
    t.pencolor(col2)
    if col2 == 'white':
        t.pencolor('black')
    t.fillcolor(col2)
    t.pd()
    t.begin_fill()
    t.circle(r)
    t.end_fill()

    # draw two 'fishes'
    t.pencolor(col1)
    t.fillcolor(col1)
    t.begin_fill()
    t.circle(r, 180)
    t.circle(r2, 180)
    t.circle(-r2, 180)
    t.end_fill()

    # black 'eye'  
    t.seth(0)
    t.pu()
    t.setpos(-(r4 + r8) + x0, y0)
    t.pd()
    t.begin_fill()
    t.circle(r8)
    t.end_fill()

    # white 'eye'
    t.pencolor(col2)
    t.fillcolor(col2)
    t.seth(0)
    t.pu()
    t.setpos(-(r+r4+r8) + x0, y0)
    t.pd()
    t.begin_fill()
    t.circle(r8)
    t.end_fill() 

    # put the turtle back where it started
    t.pu()
    t.setpos(start)
    t.seth(startheading)
    t.color(*startcolour)
 
 
if __name__ == '__main__':
    main()


'''
Main Python Turtle Commands

t.clear()       Clear screen
t.home()        Put turtle at screen centre pointing up
t.speed()       Set speed of turtle movement - has one parameter (0-10)
t.st()          Show turtle
t.ht()          Hide turtle (drawing is faster with turtle hidden)
t.setpos()      Set turtle position - has 2 parameters for x and y position
t.seth()        Set turtle heading - has 1 parameter (0-360)
t.fd()          Move forwards - has 1 parameter for distance
t.bk()          Move backward - has 1 parameter for distance
t.rt()          Turn right - has 1 parameter for number of degrees
t.lt()          Turn left  - has 1 parameter for number of degrees
t.circle()      Draw an arc - has 2 main parameters for radius and angle turned
t.pd()          Pen down
t.pu()          Pen up
t.pensize()     Pen width   - has 1 parameter for width
t.pencolor()    Pen colour  - has 1 constant or 3 RGB values as parameters
t.fillcolor()   Fill colour - has 1 constant or 3 RGB values as parameters
t.begin_fill()  Shapes drawn between this and t.end_fill are filled in
t.end_fill()    Ends filling in started by t.begin_fill
t.dot()         Put a dot at the current position
t.stamp()       Put a turtle at the current position and heading
t.write()       Writes text - details of text, alignment, font, etc. as parameters


Useful Colour Names

black, white, gray, light gray
red, dark red, maroon, salmon, pink
orange, dark orange, coral
yellow, gold, goldenrod
green, dark green, light green, olive drab
cyan, dark cyan, turquoise, aquamarine, sky blue
blue, dark blue, cornflower blue
purple, violet, magenta, orchid, slate blue
brown, sienna, chocolate, khaki, beige, tan


PMP Turtle Utils Commands

initializescreen(width=1000, height=850, colour=WHITE, title='Turtle Graphics')
initializeturtle(shape='turtle', pen=BLACK, fill=WHITE)
write(turtle, text, fontsize=14, colour=BLACK, align='center', font='Arial', face='normal')
polygon(turtle, sides, size, pen=BLACK, fill=WHITE, filled=False, centred=False, direction='R')
square(turtle, size, pen=BLACK, fill=WHITE, filled=False, centred=False, direction='R')
rectangle(turtle, width, height, pen=BLACK, fill=WHITE, filled=False, centred=False, direction='R')
segment(turtle, radius, angle, pen=BLACK, fill=WHITE, filled=False, direction='R')
sector(turtle, radius, angle, pen=BLACK, fill=WHITE, filled=False, centred=True, direction='R')
circle(turtle, radius, pen=BLACK, fill=WHITE, filled=False, centred=True, direction='R')
vesica(turtle, size [>=10], angle [30-159], pen=BLACK, fill=WHITE, filled=False) 
spiral(turtle, sides=4, size=100, gap=1, pen=BLACK, direction='R')
star(turtle, points, size, pen=BLACK, fill=WHITE, filled=False, direction='R')

'''
