import turtle
import TurtleUtilsPMP as tp


def main():
    scr = tp.initializescreen()
    t1 = tp.initializeturtle()

    scr.tracer(1000)
    # Accelerates drawing by updating screen only every n frames.

    t1.ht()
    myPoints = [[-400,-200], [0,400], [400,-200]]
    sierpinski(t1, myPoints, 5)  # '5' here is the depth of recursion.
    # A depth of 6 is the highest which shows clearly on screen (minimum = 0, maximum = 10).
    scr.exitonclick()


def sierpinski(t, points, depth):
    colormap = ['blue','red','green','cyan','yellow','violet','orange','purple','dark green','brown','black']
    # Try changing the order of the colours or replacing them with different ones.
    drawTriangle(t, points, colormap[depth])
    if depth > 0:
        sierpinski(t, [points[0], getMid(points[0], points[1]), getMid(points[0], points[2])], depth-1)
        sierpinski(t, [points[1], getMid(points[0], points[1]), getMid(points[1], points[2])], depth-1)
        sierpinski(t, [points[2], getMid(points[2], points[1]), getMid(points[0], points[2])], depth-1)


def getMid(p1, p2):
    return ((p1[0]+p2[0]) / 2, (p1[1] + p2[1]) / 2)


def drawTriangle(t, points, color):
    t.fillcolor(color)
    t.pu()
    t.setpos(points[0][0], points[0][1])
    t.pd()
    t.begin_fill()
    t.setpos(points[1][0], points[1][1])
    t.setpos(points[2][0], points[2][1])
    t.setpos(points[0][0], points[0][1])
    t.end_fill()

 
if __name__ == '__main__':
    main()


'''
Main Python Turtle Commands

t.clear()       Clear screen
t.home()        Put turtle at screen centre pointing up
t.speed()       Set speed of turtle movement - has one parameter (0-10)
t.st()          Show turtle
t.ht()          Hide turtle (drawing is faster with turtle hidden)
t.setpos()      Set turtle position - has 2 parameters for x and y position
t.seth()        Set turtle heading - has 1 parameter (0-360)
t.fd()          Move forwards - has 1 parameter for distance
t.bk()          Move backward - has 1 parameter for distance
t.rt()          Turn right - has 1 parameter for number of degrees
t.lt()          Turn left  - has 1 parameter for number of degrees
t.circle()      Draw an arc - has 2 main parameters for radius and angle turned
t.pd()          Pen down
t.pu()          Pen up
t.pensize()     Pen width   - has 1 parameter for width
t.pencolor()    Pen colour  - has 1 constant or 3 RGB values as parameters
t.fillcolor()   Fill colour - has 1 constant or 3 RGB values as parameters
t.begin_fill()  Shapes drawn between this and t.end_fill are filled in
t.end_fill()    Ends filling in started by t.begin_fill
t.dot()         Put a dot at the current position
t.stamp()       Put a turtle at the current position and heading
t.write()       Writes text - details of text, alignment, font, etc. as parameters


Useful Colour Names

black, white, gray, light gray
red, dark red, maroon, salmon, pink
orange, dark orange, coral
yellow, gold, goldenrod
green, dark green, light green, olive drab
cyan, dark cyan, turquoise, aquamarine, sky blue
blue, dark blue, cornflower blue
purple, violet, magenta, orchid, slate blue
brown, sienna, chocolate, khaki, beige, tan


PMP Turtle Utils Commands

initializescreen(width=1000, height=850, colour='white', title='Turtle Graphics')
initializeturtle(shape='turtle', pen='black', fill='white')
write(turtle, text, fontsize=14, colour='black', align='center', font='Arial', face='normal')
polygon(turtle, sides, size, pen='black', fill='white', filled=False, centred=False, lr='R')
square(turtle, size, pen='black', fill='white', filled=False, centred=False, lr='R')
rectangle(turtle, width, height, pen='black', fill='white', filled=False, centred=False,
    lr='R', oval=False)
oval(turtle, width, height, pen='black', fill='white', filled=False, centred=False, lr='R')
segment(turtle, radius, angle, pen='black', fill='white', filled=False, lr='R')
sector(turtle, radius, angle, pen='black', fill='white', filled=False, centred=True, lr='R')
circle(turtle, radius, pen='black', fill='white', filled=False, centred=True, lr='R')
ellipse(turtle, long radius, short radius, sides=20, pen='black', fill='white',
    filled=False, centred=False, lr='R', semi=False)
vesica(turtle, size [>=10], angle [30-159], pen='black', fill='white', filled=False) 
spiral(turtle, sides=4, size=100, gap=1, pen='black', lr='R')
star(turtle, points, size, pen='black', fill='white', filled=False, lr='R')
crescent(turtle, size [>=10], angle [30-120], pen='black', fill='white', filled=False, lr='R')

'''
