import turtle
import TurtleUtilsPMP as tp
import math as m
import random as r


def main():
    scr = tp.initializescreen()
    # 1000, 850, 'white', 'Turtle Graphics' are defaults for width, height, colour, and title
    ada = tp.initializeturtle()
    # 'turtle', 'black', 'white' are defaults for shape, colour, and fill

    ada.speed(0)
    ada.ht()
    scr.tracer(1000)
    # Accelerates drawing by updating screen only every n frames.

    ada.pu()
    ada.setpos(-300, 150)
    ada.seth(0)
    ada.pd()
    tree1(ada, 10, 'green', 'brown', 4, 5)
    ada.pu()
    ada.setpos(250, 150)
    ada.seth(0)
    ada.pencolor('dark green')
    ada.fillcolor('green')
    ada.pd()
    tree2(ada, 65, 6)
    ada.pu()
    ada.setpos(375, -375)
    ada.seth(0)
    ada.pencolor('forest green')
    ada.pd()
    fern1(ada, 60, 2)
    ada.pu()
    ada.setpos(200, -175)
    ada.seth(0)
    ada.pencolor('olive drab')
    ada.pd()
    fern2(ada, 60)
    ada.pu()
    ada.setpos(-250, -375)
    ada.seth(0)
    ada.pd()
    tree3(ada, 20, 20, 12, 'dark green', 'sienna', 5)
    ada.pu()
    ada.setpos(-150, -375)
    ada.seth(0)
    ada.pd()
    tree3(ada, 10, 25, 10, 'green yellow', 'goldenrod', 5, 3.5)
    ada.pu()
    ada.setpos(-450, -400)
    ada.seth(0)
    ada.st()
    
    scr.exitonclick()


def tree1(t, level=7, col1='green', col2='brown', colchange=3, scale=5):
# s is complexity (best 7-12)
    if level < 1:
        return
    t.pencolor(col2)
    if level <= colchange:
        t.pencolor(col1)
    s = scale
    c1 = col1
    c2 = col2
    t.pensize(int(level * s / 5) + 1)
    t.fd(level * s)
    t.rt(30)
    tree1(t, level - 1, c1, c2, scale=s)
    t.lt(60)
    tree1(t, level - 1, c1, c2, scale=s)
    t.rt(30)
    t.pencolor(col2)
    if level <= colchange:
        t.pencolor(col1)
    t.bk(level * s)


def tree2(t, size=100, level=10, pensz=10):
# level is best set at less than size/10 and should always be less than size/2
    if size < level:
        return
    t.pensize(int(pensz / 2) + 1)
    t.begin_fill()
    for i in range(4):
        t.fd(size)
        t.rt(90)
    t.end_fill()
    t.fd(size)
    t.lt(45)
    tree2(t, size / m.sqrt(2), level, pensz - 1)
    t.pu()
    t.rt(90)
    t.fd(size / m.sqrt(2))
    t.pd()
    tree2(t, size / m.sqrt(2), level, pensz - 1)
    t.pu()
    t.bk(size / m.sqrt(2))
    t.lt(45)
    t.pd()
    t.bk(size)


def tree3(t, height=20, width=20, level=10, col1='green', col2='brown', colchange=4, scale=5):
# height (best 15-25), width (best similar to height), level (best 10-15)
    if level == 0:
        return
    t.pencolor(col2)
    if level <= colchange:
        t.pencolor(col1)
    s = scale
    c1 = col1
    c2 = col2
    t.pensize(int(level * s / 10) + 1)
    t.fd(height * s)
    t.rt(width)
    tree3(t, height * (0.5 + 0.04 * r.randint(1, 10)), width, level - 1, c1, c2, scale=s)
    t.lt(2 * width)
    tree3(t, height * (0.5 + 0.04 * r.randint(1, 10)), width, level - 1, c1, c2, scale=s)
    t.rt(width)
    t.pencolor(col2)
    if level <= colchange:
        t.pencolor(col1)
    t.bk(height * s)


def fern1(t, size=50, curve=0):   # curve=0 gives a straight fern.
    if size < 0.16:
        return
    t.pensize(int(size / 8) + 1)  # Set stem width.
    t.lt(curve)
    t.fd(size)
    t.rt(90)
    fern1(t, size * 0.3, curve)   # Draw right-hand sub-fern.
    t.rt(180)
    fern1(t, size * 0.3, curve)   # Draw left-hand sub-fern.
    t.rt(90)
    fern1(t, size * 0.85, curve)  # Draw next part of main fern.
    t.bk(size)
    t.rt(curve)


def fern2(t, size=50):
    minDist = 0.3
    widthFactor = 0.1
    stemFactor = 0.04
    restFactor = 0.85
    branchFactor = 0.3
    stemSegs = 5        # Sets the curvature of the fern(1 is straight).
    stemSegAngle = 1
    scale = 25

    if size < minDist:
        return
    t.pensize(int(scale * size * stemFactor * widthFactor) + 1)
    # Width of the stem is a fraction of the length.

    # Draw the stem.
    for i in range(stemSegs):
        t.fd(scale * size * stemFactor / stemSegs)
        if i < (stemSegs - 1):
            t.lt(stemSegAngle)
            
    # Draw the right-hand sub-fern.
    t.rt(90)
    fern2(t, size * branchFactor)

    # Draw the left-hand sub-fern.
    t.rt(180)
    fern2(t, size * branchFactor)

    # Draw the next part of the main fern.
    t.rt(90)
    fern2(t, size * restFactor)

    # Draw back down to the start of this sub-fern with the original thickness.
    t.pensize(int(scale * size * stemFactor * widthFactor) + 1)
    for i in range(stemSegs):
        t.bk(scale * size * stemFactor / stemSegs)
        if i < (stemSegs - 1):
            t.rt(stemSegAngle)


if __name__ == '__main__':
    main()


'''
Main Python Turtle Commands

t.clear()       Clear screen
t.home()        Put turtle at screen centre pointing up
t.speed()       Set speed of turtle movement - has one parameter (0-10)
t.st()          Show turtle
t.ht()          Hide turtle (drawing is faster with turtle hidden)
t.setpos()      Set turtle position - has 2 parameters for x and y position
t.seth()        Set turtle heading - has 1 parameter (0-360)
t.fd()          Move forwards - has 1 parameter for distance
t.bk()          Move backward - has 1 parameter for distance
t.rt()          Turn right - has 1 parameter for number of degrees
t.lt()          Turn left  - has 1 parameter for number of degrees
t.circle()      Draw an arc - has 2 main parameters for radius and angle turned
t.pd()          Pen down
t.pu()          Pen up
t.pensize()     Pen width   - has 1 parameter for width
t.pencolor()    Pen colour  - has 1 constant or 3 RGB values as parameters
t.fillcolor()   Fill colour - has 1 constant or 3 RGB values as parameters
t.begin_fill()  Shapes drawn between this and t.end_fill are filled in
t.end_fill()    Ends filling in started by t.begin_fill
t.dot()         Put a dot at the current position
t.stamp()       Put a turtle at the current position and heading
t.write()       Writes text - details of text, alignment, font, etc. as parameters


Useful Colour Names

black, white, gray, light gray
red, dark red, maroon, salmon, pink
orange, dark orange, coral
yellow, gold, goldenrod
green, dark green, light green, olive drab
cyan, dark cyan, turquoise, aquamarine, sky blue
blue, dark blue, cornflower blue
purple, violet, magenta, orchid, slate blue
brown, sienna, chocolate, khaki, beige, tan


PMP Turtle Utils Commands

initializescreen(width=1000, height=850, colour='white', title='Turtle Graphics')
initializeturtle(shape='turtle', pen='black', fill='white')
write(turtle, text, fontsize=14, colour='black', align='center', font='Arial', face='normal')
polygon(turtle, sides, size, pen='black', fill='white', filled=False, centred=False, lr='R')
square(turtle, size, pen='black', fill='white', filled=False, centred=False, lr='R')
rectangle(turtle, width, height, pen='black', fill='white', filled=False, centred=False,
    lr='R', oval=False)
oval(turtle, width, height, pen='black', fill='white', filled=False, centred=False, lr='R')
segment(turtle, radius, angle, pen='black', fill='white', filled=False, lr='R')
sector(turtle, radius, angle, pen='black', fill='white', filled=False, centred=True, lr='R')
circle(turtle, radius, pen='black', fill='white', filled=False, centred=True, lr='R')
ellipse(turtle, long radius, short radius, sides=20, pen='black', fill='white',
    filled=False, centred=False, lr='R', semi=False)
vesica(turtle, size [>=10], angle [30-159], pen='black', fill='white', filled=False) 
spiral(turtle, sides=4, size=100, gap=1, pen='black', lr='R')
star(turtle, points, size, pen='black', fill='white', filled=False, lr='R')
crescent(turtle, size [>=10], angle [30-120], pen='black', fill='white', filled=False, lr='R')

'''
